open Map;;
open Analyse;;
open Printf;;

module Nomsommet:OrderedType =
  struct
    type t = char list

    let rec compare a b = match a with
      |[]     -> 1
      |ta::qa -> match b with
		 |[]     -> -1
		 |tb::qb -> let va = int_of_char ta in
			    let vb = int_of_char tb in
			    if va <> vb then (va-vb)
			    else compare qa qb

  end
    
module Sommets = Map.Make(Nomsommet);;
  
let nouvgraph = Sommets.empty;;

let nouvarc = Sommets.empty;;

let ajoutsommet graphe sommet = Sommets.add sommet nouvarc graphe;;

let sousajoutarc graphe dep dest poids = Sommets.add dep (if   Sommets.mem dep graphe
							  then Sommets.add dest poids (Sommets.find dep graphe)
							  else Sommets.add dest poids nouvarc)

						          graphe;;

let ajoutarc graphe som1 som2 poids = let g = sousajoutarc graphe som1 som2 poids in
				      sousajoutarc g som2 som1 poids;;
  
let poids graphe dep dest = Sommets.find dest (Sommets.find dep graphe);;				      

let rec eliminator g som  = if Sommets.is_empty g
			    then g
			    else let s = Sommets.choose g in
				 if Sommets.mem som (snd s)
				 then let nouvarcs   = Sommets.remove som (snd s) in
				      let nouvgraphe = Sommets.remove (fst s) g in
				      let nouvclef   = (fst s) in
				      Sommets.add nouvclef nouvarcs (eliminator nouvgraphe som)
	 			 else let nouvgraphe = Sommets.remove (fst s) g in
				      Sommets.add (fst s) (snd s) (eliminator nouvgraphe som);;
  
let supprimesommet g som = let h = Sommets.remove som g in
			   eliminator h som;;

let rec verseur g1 g2 som = if Sommets.is_empty g1
			    then g2
			    else let s = Sommets.choose g1 in
				 if Sommets.mem som (snd s)
				 then let p  = poids g1 (fst s) som in
				      let h1 = Sommets.remove (fst s) g1 in
				      let h2 = ajoutarc g2 (fst s) som p in
				      verseur g1 g2 som
				 else let h1 = Sommets.remove (fst s) g1 in
				      verseur h1 g2 som;;				      
					       
let transvasesommet g1 g2 som = let h1 = supprimesommet g1 som in
				let h2 = verseur (Sommets.remove som g1) g2 som in
				(h1,h2);;

let concat clef valeur accumulateur = Sommets.add clef Pervasives.max_int accumulateur;;
  
let initdistances graphe = Sommets.fold concat graphe nouvgraph;;

let petit a b c = if b <= c then a else c;;

let min a b = if a <= b then a else b;;

let rec majd d i a = if Sommets.is_empty d
		     then d
		     else let sommet = Sommets.choose d in
		  	  if Sommets.mem (fst sommet) i
			  then if   Sommets.mem a (Sommets.find (fst sommet) i)			
			       then let p = poids i (fst sommet) a in
				    let newval = min (snd sommet) (p + (snd (Sommets.find a d))) in
				    let e = Sommets.remove (fst sommet) d in
				    Sommets.add (fst sommet) newval (majd e i a)
			       else let e = Sommets.remove (fst sommet) d in
				    Sommets.add (fst sommet) (snd sommet) (majd e i a);;
							   					
let rec boucle p d o g = if (g == Sommets.empty)
			 then snd (Sommets.find o d)
		         else let a = Sommets.fold petit d (Sommets.choose d) in
			      let i = Sommets.remove a g in
			      let (q,h) = transvasesommet p g a in
			      let e = majd d i a in
			      boucle q e o h;;
    
let dijkstra graphe dep dest = boucle nouvgraph (initdistances graphe) dep graphe;;
		
  (*
let rec stringoftrip t = match t with
  |(a,b,c) -> a^b^(string_of_int c)
  |_       -> "";;

let stringofcouple c = match c with
  |(a,b) -> "("^(string_of_int a)^","^(string_of_int b)^")"
  |_     -> "";;
  
let rec stringoflist l = match l with
	       |[]   -> "]"
	       |a::b -> (stringoftrip a)^","^(stringoflist b);;

let stringofforest f = match f with
  |(l,c) -> "["^(stringoflist l)^(stringofcouple c);;
  
let a = printf (stringofforest (analyse_file_1 1.txt));;

   *)
   
